#!/bin/bash

COMPOSE_PATH="/etc/docker/compose"
CUSTOM_ENV_FILE="custom.env"

INSTALLATION_TYPE="CC"
XIVOCC_CONFIG_SCRIPT="/usr/bin/xivocc-configure-xivopbx"
RESTART_REPLY=false

install_package_on_xivo() {
    local xivo_host="${1}"
    ssh "root@${xivo_host}" 'DEBIAN_FRONTEND=noninteractive apt-get install --yes -o Dpkg::Options::="--force-confnew" xivocc-docker-components'
}

launch_xivo_configuration_script() {
    local xivo_host="${1}"
    echo -e "\e[1;33mXiVO PBX Configuration...\e[0m"

    # Run the xuc setup script on XiVO PBX
    # shellcheck disable=SC2029
    ssh "root@${xivo_host}" "bash $XIVOCC_CONFIG_SCRIPT \
        -h $XUC_HOST \
        -r $RESTART_REPLY \
        -s $XIVO_AMI_SECRET \
        -t $INSTALLATION_TYPE"
}

check_if_xivo_configured() {
    local xivo_host="${1}"
    local wizard_state
    wizard_state=$(curl -sk -X GET --header 'Accept: application/json' "https://${xivo_host}:9486/1.1/wizard")

    if echo "${wizard_state}" | grep -q '"configured"[[:space:]]*:[[:space:]]*true'; then
        echo -e "\e[1;32mYour Xivo is configured\e[0m"
        return 0
    else
        echo -e "\e[1;31mYour Xivo is not configured\e[0m"
        return 1
    fi
}

set_usm_amqp_pwd_in_custom_env_file() {
    local xivo_host="${1}"
    local get_usm_event_amqp_secret
    get_usm_event_amqp_secret=$(ssh "root@${xivo_host}" "grep -oP -m 1 'USM_EVENT_AMQP_SECRET=\K.*' /etc/docker/xivo/custom.env")

    if [ "${get_usm_event_amqp_secret}" != "${USM_EVENT_AMQP_SECRET}" ] && [ "${get_usm_event_amqp_secret}" != "default" ]; then
        echo "USM_EVENT_AMQP_SECRET=${get_usm_event_amqp_secret}" >>${COMPOSE_PATH}/${CUSTOM_ENV_FILE}
    elif [ "${get_usm_event_amqp_secret}" = "default" ]; then
        echo -e "\\e[1;33mWARN: USM_EVENT_AMQP_SECRET was not found in XiVO configuration\\e[0m"
        echo -e "\\e[1;33m Please re-run the following commmand when your XiVO is upgraded:\\e[0m"
        echo -e ""
        echo -e "\\e[1;33m    echo \$(ssh root@${xivo_host} \"grep -oP -m 1 'USM_EVENT_AMQP_SECRET=.*' /etc/docker/xivo/custom.env\") >> ${COMPOSE_PATH}/${CUSTOM_ENV_FILE} \\e[0m"
        echo -e ""
        echo -e ""
    fi
}

check_if_xivo_is_ssh_reachable() {
    local xivo_host="${1}"
    local ssh_state
    ssh_state=$(ssh -o PasswordAuthentication=no -o ConnectTimeout=3 "root@${xivo_host}" "echo true")

    if $ssh_state; then
        echo -e "\e[1;32mYour Xivo is reachable via SSH\e[0m"
        return 0
    else
        echo -e "\e[1;31mYour Xivo is not reachable via SSH with ssh keys, check your SSH config\e[0m"
        return 1
    fi
}

source_custom_env_file() {
    # shellcheck disable=SC1090
    source ${COMPOSE_PATH}/${CUSTOM_ENV_FILE}

    # Check if there's value, else, will add default
    XIVO_HOST=${XIVO_HOST:-127.0.0.1}
    XUC_HOST=${XUC_HOST:-127.0.0.1}
    ELASTICSEARCH_HOST=${ELASTICSEARCH_HOST:-127.0.0.1}
    CONFIGURE_REPLY=${CONFIGURE_REPLY:-false}
    WEEKS_TO_KEEP=${WEEKS_TO_KEEP:-52}
    RECORDING_WEEKS_TO_KEEP=${RECORDING_WEEKS_TO_KEEP:-52}
    RESTART_REPLY=${RESTART_REPLY:-false}
    USM_EVENT_AMQP_SECRET=${USM_EVENT_AMQP_SECRET:-default}
}

xivocc_configures_xivo_post_wizard() {
    if check_if_xivo_configured "${XIVO_HOST}"; then
        if check_if_xivo_is_ssh_reachable "${XIVO_HOST}"; then
            install_package_on_xivo "${XIVO_HOST}"
            launch_xivo_configuration_script "${XIVO_HOST}"
            set_usm_amqp_pwd_in_custom_env_file "${XIVO_HOST}"
            set_call_quality_stats_component_secrets_in_custom_env_file "${XIVO_HOST}"
        else
            echo -e "\e[1;31mXiVO PBX is not reachable via ssh with key. Exiting...\e[0m"
            exit 1
        fi
    else
        echo -e "\e[1;31mXiVO PBX is not configured. Exiting...\e[0m"
        exit 1
    fi
}
